@@ -75,12 +75,12 @@ XdgMenuPrivate::XdgMenuPrivate(XdgMenu *parent):
     mRebuildDelayTimer.setSingleShot(true);
     mRebuildDelayTimer.setInterval(REBUILD_DELAY);
 
-    connect(&mRebuildDelayTimer, SIGNAL(timeout()), this, SLOT(rebuild()));
-    connect(&mWatcher, SIGNAL(fileChanged(QString)), &mRebuildDelayTimer, SLOT(start()));
-    connect(&mWatcher, SIGNAL(directoryChanged(QString)), &mRebuildDelayTimer, SLOT(start()));
+    connect(&mRebuildDelayTimer, &QTimer::timeout, this, &XdgMenuPrivate::rebuild);
+    connect(&mWatcher, &QFileSystemWatcher::fileChanged, &mRebuildDelayTimer, QOverload<>::of(&QTimer::start));
+    connect(&mWatcher, &QFileSystemWatcher::directoryChanged, &mRebuildDelayTimer, QOverload<>::of(&QTimer::start));
 
 
-    connect(this, SIGNAL(changed()), q_ptr, SIGNAL(changed()));
+    connect(this, &XdgMenuPrivate::changed, q_ptr, &XdgMenu::changed);
 }
 
 
@@ -410,14 +410,18 @@ QDomElement XdgMenu::findMenu(QDomElement& baseElement, const QString& path, boo
         return QDomElement();
 
 
+#if (QT_VERSION >= QT_VERSION_CHECK(5,15,0))
+    const QStringList names = path.split(QLatin1Char('/'), Qt::SkipEmptyParts);
+#else
     const QStringList names = path.split(QLatin1Char('/'), QString::SkipEmptyParts);
+#endif
     QDomElement el = baseElement;
-    for (const QString &name : names)
+    for (const QString &n : names)
     {
         QDomElement p = el;
         el = d->mXml.createElement(QLatin1String("Menu"));
         p.appendChild(el);
-        el.setAttribute(QLatin1String("name"), name);
+        el.setAttribute(QLatin1String("name"), n);
     }
     return el;
 
